px = 0.027;
window_sep = 120*px;
layer = 0.050;
membranethickness = 0.4*layer;
membrane_xseparation = 35*px;

xbulk = 300*px;
ybulk = 330*px;
zbulk = 70*layer;

d_bottom_volume = 40*px;
h_bottom_volume = 0.06;
d_top_volume = 40*px;
h_top_volume = 0.1;

d_bottom_pump = 40*px;
h_bottom_pump = 0.08;
d_top_pump = 40*px;
h_top_pump = 0.08;
memb_pump = 0.4*layer;

xy_vert_channel = 6*px;
dpinhole = 42*px;
odpinhole = 40*px;
idpinhole = 20*px;

channel_sep = 4*px;

l_outlet = 100*px;


l_hrzt_channel = 38*px;
l_hrzt_channel2 = 40*px;
l_hrzt_channel3 = d_bottom_volume/2 - xy_vert_channel/2 - channel_sep/2 + 10*px + xy_vert_channel/2;
h_hrzt_channel = 0.15;

module bulk() {
  cube(size=[xbulk, ybulk, zbulk], center=false);
}

module top_volume(d, h) {
    color("red") cylinder(d=d, h=h, $fn=3.14*d/0.01, center=false);
}

module bottom_volume(d, h) {
    color("dodgerblue") cylinder(d=d, h=h, $fn=3.14*d/0.01, center=false);
}

module vert_channel(z) {
    translate([-xy_vert_channel/2, -xy_vert_channel/2, 0]) 
        cube([xy_vert_channel, xy_vert_channel, z]);
}

module hrzt_channelx(l) {
    translate([0, -xy_vert_channel/2, 0]) cube([l, xy_vert_channel, h_hrzt_channel]);
}

module hrzt_channely(l) {
    translate([-xy_vert_channel/2, 0, 0]) cube([xy_vert_channel, l, h_hrzt_channel]);
}

module pinhole(l=3) {
    color("white") cylinder(d=dpinhole, h=l, $fn=3.14*dpinhole/0.01, center=false);
}

module reservoir() {
    color("pink") cylinder(d=40*px, h=2, $fn=3.14*dpinhole/0.01, center=false);
}

module outlet_channel() {
    translate([-10*px, 0, 0])
        color("cyan") mirror([0,0,1]) cube([20*px, l_outlet, 0.5+0.01]);
}

module channel_transition() {
    x1 = xy_vert_channel;
    z1 = h_hrzt_channel;
    steps = 7;
    for(i=[2:steps]) {
        color("cyan") translate([-(x1+(i-1)*2*px)/2, px*(i-2)*3, 0]) cube([x1+(i-1)*2*px, 3*px, z1+(i-1)*0.05]);
    }
}

module valve(theta=0) {
    bottom_volume(d=d_bottom_volume, h=h_bottom_volume);
    translate([0, 0, h_bottom_volume+membranethickness]) 
        top_volume(d=d_top_volume, h=h_top_volume);
    
    // top volume channels        
    rotate([0, 0, theta]) {
        translate([0, d_top_volume/2-xy_vert_channel/2, h_top_volume+h_bottom_volume+membranethickness]) {
            vert_channel(z=0.1+h_hrzt_channel);
        }
        mirror([0,1,0]) {
            translate([0, d_top_volume/2-xy_vert_channel/2, h_top_volume+h_bottom_volume+membranethickness]) {
                vert_channel(z=0.1+h_hrzt_channel);
            }
        }
    }
    
    // bottom volume channels
    translate([xy_vert_channel/2+channel_sep/2, 0, -0.2-h_hrzt_channel]) {
        vert_channel(z=0.2+h_hrzt_channel); 
    }
    
    mirror([1,0,0]) { 
        translate([xy_vert_channel/2+channel_sep/2, 0, -0.2-h_hrzt_channel]) {
            vert_channel(z=h_hrzt_channel+0.2); 
        }
    }
}

module pump() {
    bottom_volume(d=d_bottom_pump, h=h_bottom_pump);
    translate([0, 0, h_bottom_pump+membranethickness]) 
        top_volume(d=d_top_pump, h=h_top_pump);
    
    // top volume channels
    translate([0, d_top_volume/2-xy_vert_channel/2, h_top_pump+h_bottom_pump+memb_pump])
        vert_channel(z=0.1+h_hrzt_channel);
    
    mirror([0,1,0])
    translate([0, d_top_volume/2-xy_vert_channel/2, h_top_pump+h_bottom_pump+membranethickness])
        vert_channel(z=0.1+h_hrzt_channel);
    
    // bottom volume channels
    for(i=[1:2]) {
        rotate([0, 0, 180*i])
            translate([0, d_bottom_pump/2-xy_vert_channel/2, -0.2-h_hrzt_channel])
                vert_channel(z=0.2+h_hrzt_channel); 
    }
    
    translate([13*px, 9*px, -0.2-h_hrzt_channel])
        vert_channel(z=0.2+h_hrzt_channel); 
    
    translate([-13*px, 9*px, -0.2-h_hrzt_channel])
        vert_channel(z=0.2+h_hrzt_channel); 
    
    translate([-13*px, -9*px, -0.2-h_hrzt_channel])
        vert_channel(z=0.2+h_hrzt_channel); 
    
    translate([13*px, -9*px, -0.2-h_hrzt_channel])
        vert_channel(z=0.2+h_hrzt_channel); 
}

valveH = h_hrzt_channel+ 0.1 + h_top_volume + h_bottom_volume + membranethickness + h_hrzt_channel + 0.2;
dispH = h_hrzt_channel+ 0.1 + h_top_pump + h_bottom_pump + memb_pump + h_hrzt_channel + 0.2;
//valveW = dpinhole + l_hrzt_channel*2 + xy_vert_channel + channel_sep;

module device() {
    //bottom left
    translate([d_bottom_pump/2+d_bottom_volume/2+10*px, 0, 0]) {
        rotate([0,0,90]) valve();
        //fluid channel        
        translate([0, -channel_sep/2, -0.2-h_hrzt_channel]) {
            mirror([0,1,0]) hrzt_channely(13*px+d_bottom_volume/2);
            translate([xy_vert_channel/2,-(13*px+d_bottom_volume/2),0]) {
                mirror([1,0,0]) hrzt_channelx(10*px+d_bottom_volume/2+d_bottom_pump/2+xy_vert_channel);
                translate([-(10*px+d_bottom_volume/2+d_bottom_pump/2+xy_vert_channel/2), 0, 0])
                    hrzt_channely(13*px+xy_vert_channel/2);
            }
        }
        translate([0, channel_sep/2+xy_vert_channel/2, 0]) {
            mirror([0,0,1]) vert_channel(0.35+0.3+h_hrzt_channel);
            translate([0, 0, -(0.65+h_hrzt_channel)]) {
                hrzt_channely(10*px);
                translate([0, 10*px, h_hrzt_channel]) {
                    mirror([0,0,1]) channel_transition();
                    translate([0, 18*px, 0]) {
                        outlet_channel();
                        translate([0, l_outlet+18*px, 0]) {
                            mirror([0,0,1]) mirror([0,1,0]) channel_transition();
                            translate([0, 0, -h_hrzt_channel]) {
                                hrzt_channely(10*px);
                                translate([0, 10*px, 0]) {
                                    vert_channel(1);
                                    translate([0, 0, 0.2+h_hrzt_channel])
                                        pinhole(zbulk-0.7+0.01);
                                }
                            }
                        }
                    }
                }
            }
        }
        //air channel
        mirror([0,1,0])
        translate([d_top_volume/2-xy_vert_channel/2, 0, valveH-0.2-h_hrzt_channel*2]) {
            hrzt_channelx(10*px+dpinhole/2+xy_vert_channel/2);
            translate([10*px+dpinhole/2+xy_vert_channel/2, 0, 0]) {
                vert_channel(z=1);
                translate([0, 0, 0.2+h_hrzt_channel])
                    pinhole(zbulk-(0.8+valveH+0.2)+0.01);
            }
        }
        translate([-(d_top_volume/2-xy_vert_channel/2), 0, h_bottom_volume+h_top_volume+membranethickness+0.1])
            mirror([0,1,0]) hrzt_channely(120*px+0.01);
    }
    
    //top left
    translate([d_bottom_pump/2+d_bottom_volume/2+10*px, 0, valveH]) {
        rotate([0,0,90]) valve(theta=90);
        //fluid channel
        translate([0, channel_sep/2, -0.2-h_hrzt_channel]) {
            hrzt_channely(d_bottom_volume/2-channel_sep/2+10*px+xy_vert_channel/2);
            translate([0, d_bottom_volume/2-channel_sep/2+10*px+xy_vert_channel/2, -valveH]) {
                vert_channel(z=valveH+h_hrzt_channel);
                mirror([1,0,0]) hrzt_channelx(d_bottom_pump/2+d_bottom_volume/2+10*px);
                mirror([0,1,0]) translate([-(d_bottom_pump/2+d_bottom_volume/2+10*px), -xy_vert_channel/2, 0])
                    hrzt_channely(10*px+2*xy_vert_channel);
            }
        }
        
        translate([0, -channel_sep/2, -0.2-h_hrzt_channel]) {
            mirror([0,1,0]) hrzt_channely(d_bottom_volume/2-channel_sep/2);
            translate([0,-(d_bottom_volume/2-channel_sep/2-xy_vert_channel/2),0]) {
                hrzt_channelx(20*px);
                translate([20*px-xy_vert_channel/2, 0, 0]) {
                    mirror([0,1,0]) hrzt_channely(50*px+xy_vert_channel/2);
                    translate([0, -(50*px+xy_vert_channel/2), h_hrzt_channel]) {
                        mirror([0,0,1]) vert_channel(0.49+h_hrzt_channel);
                        translate([0,0,-0.2-h_hrzt_channel]) {
                            *mirror([0,0,1]) reservoir();
                            translate([0, -10*px, -0.3]) {
                                rotate([90,0,0]) pinhole(l=1.1);
                                translate([0, 0, -0.07])
                                    hrzt_channely(10*px+xy_vert_channel/2);
                            }
                        }
                    }
                }
            }
        }
        //air channel
        translate([0, d_top_volume/2-xy_vert_channel/2, h_bottom_volume+h_top_volume+membranethickness]) {
            vert_channel(1);
            translate([0, 0, 0.4])
                pinhole(l=zbulk-(0.8+valveH*2)-0.14);
        }
        translate([0, -(d_top_volume/2), h_bottom_volume+h_top_volume+membranethickness+0.1]) {
            mirror([0,1,0]) hrzt_channely(120*px-d_top_volume/2+0.01);
        }
    }
    
    //bottom right
    mirror([1,0,0]) 
    translate([d_bottom_pump/2+d_bottom_volume/2+10*px, 0, 0]) {
        rotate([0,0,90]) valve();
        //fluid channel
        translate([0, -channel_sep/2, -0.2-h_hrzt_channel]) {
            mirror([0,1,0]) hrzt_channely(13*px+d_bottom_volume/2);
            translate([xy_vert_channel/2,-(13*px+d_bottom_volume/2),0]) {
                mirror([1,0,0]) hrzt_channelx(10*px+d_bottom_volume/2+d_bottom_pump/2+xy_vert_channel);
                translate([-(10*px+d_bottom_volume/2+d_bottom_pump/2+xy_vert_channel/2), 0, 0])
                    hrzt_channely(13*px+xy_vert_channel/2);
            }
        }
        translate([0, channel_sep/2+xy_vert_channel/2, 0]) {
            mirror([0,0,1]) vert_channel(0.35+0.3+h_hrzt_channel);
            translate([0, 0, -(0.65+h_hrzt_channel)]) {
                hrzt_channely(10*px);
                translate([0, 10*px, h_hrzt_channel]) {
                    mirror([0,0,1]) channel_transition();
                    translate([0, 18*px, 0]) {
                        outlet_channel();
                        translate([0, l_outlet+18*px, 0]) {
                            mirror([0,0,1]) mirror([0,1,0]) channel_transition();
                            translate([0, 0, -h_hrzt_channel]) {
                                hrzt_channely(10*px);
                                translate([0, 10*px, 0]) {
                                    vert_channel(1);
                                    translate([0, 0, 0.2+h_hrzt_channel])
                                        pinhole(zbulk-0.7+0.01);
                                }
                            }
                        }
                    }
                }
            }
        }
        //air channel
        mirror([0,1,0])
        translate([d_top_volume/2-xy_vert_channel/2, 0, valveH-0.2-h_hrzt_channel*2]) {
            hrzt_channelx(10*px+dpinhole/2+xy_vert_channel/2);
            translate([10*px+dpinhole/2+xy_vert_channel/2, 0, 0]) {
                vert_channel(z=1);
                translate([0, 0, 0.2+h_hrzt_channel])
                    pinhole(zbulk-(0.8+valveH+0.2)+0.01);
            }
        }
        translate([-(d_top_volume/2-xy_vert_channel/2), 0, h_bottom_volume+h_top_volume+membranethickness+0.1])
            mirror([0,1,0]) hrzt_channely(120*px+0.01);
    }
    
    //top right
    mirror([1,0,0])
    translate([d_bottom_pump/2+d_bottom_volume/2+10*px, 0, valveH]) {
        rotate([0,0,90]) valve(theta=90);
        //fluid channel
        translate([0, channel_sep/2, -0.2-h_hrzt_channel]) {
            hrzt_channely(d_bottom_volume/2-channel_sep/2+10*px+xy_vert_channel/2);
            translate([0, d_bottom_volume/2-channel_sep/2+10*px+xy_vert_channel/2, -valveH]) {
                vert_channel(z=valveH+h_hrzt_channel);
                mirror([1,0,0]) hrzt_channelx(d_bottom_volume/2);
                translate([xy_vert_channel/2-d_bottom_volume/2, 0, 0]) {
                    mirror([0,1,0]) hrzt_channely(19*px+xy_vert_channel+d_bottom_pump/2);
                    translate([0, -(19*px+xy_vert_channel/2+d_bottom_pump/2), 0])
                        mirror([1,0,0]) hrzt_channelx(xy_vert_channel/2+17*px);
                    mirror([0,0,1]) vert_channel(0.2+h_hrzt_channel);
                    translate([0, 0, -(0.2+h_hrzt_channel)]) {
                        mirror([1,0,0]) hrzt_channelx(10*px+xy_vert_channel/2+d_bottom_pump/2+13*px);
                        translate([-(10*px+xy_vert_channel/2+d_bottom_pump/2+13*px), xy_vert_channel/2, 0]) {
                            mirror([0,1,0]) hrzt_channely(21*px++xy_vert_channel);
                            translate([0, -(21*px++xy_vert_channel), 0])
                                vert_channel(0.2+h_hrzt_channel);
                        }
                    }
                }
            }
        }
        translate([0, -channel_sep/2, -0.2-h_hrzt_channel]) {
            mirror([0,1,0]) hrzt_channely(d_bottom_volume/2-channel_sep/2);
            translate([0,-(d_bottom_volume/2-channel_sep/2-xy_vert_channel/2),0]) {
                hrzt_channelx(20*px);
                translate([20*px-xy_vert_channel/2, 0, 0]) {
                    mirror([0,1,0]) hrzt_channely(50*px+xy_vert_channel/2);
                    translate([0, -(50*px+xy_vert_channel/2), h_hrzt_channel]) {
                        mirror([0,0,1]) vert_channel(0.49+h_hrzt_channel);
                        translate([0,0,-0.2-h_hrzt_channel]) {
                            *mirror([0,0,1]) reservoir();
                            translate([0, -10*px, -0.3]) {
                                rotate([90,0,0]) pinhole(l=1.1);
                                translate([0, 0, -0.07])
                                    hrzt_channely(10*px+xy_vert_channel/2);
                            }
                        }
                    }
                }
            }
        }
        //air channel
        translate([0, d_top_volume/2-xy_vert_channel/2, h_bottom_volume+h_top_volume+membranethickness]) {
            vert_channel(1);
            translate([0, 0, 0.4])
                pinhole(l=zbulk-(0.8+valveH*2)-0.14);
        }
        translate([0, -(d_top_volume/2), h_bottom_volume+h_top_volume+membranethickness+0.1]) {
            mirror([0,1,0]) hrzt_channely(120*px-d_top_volume/2+0.01);
        }
    }
    
    //center top
    translate([0, 0, dispH]) {
        valve();
        //fluid channel
        translate([channel_sep/2, 0, -(0.2+h_hrzt_channel)]) {
            hrzt_channelx(d_top_volume/2-channel_sep/2);
            translate([d_top_volume/2-channel_sep/2-xy_vert_channel/2, 0, 0]) {
                mirror([0,1,0]) hrzt_channely(d_top_pump/2+xy_vert_channel);
                translate([0, -(d_top_pump/2+xy_vert_channel/2), 0]) {
                    mirror([0,0,1]) vert_channel(dispH+0.2+h_hrzt_channel);
                    translate([0, 0, -dispH]) {
                        hrzt_channely(d_bottom_pump/2+xy_vert_channel-9*px);
                    }
                    translate([0, 0, -(dispH+0.2+h_hrzt_channel)]) {
                        mirror([1,0,0]) hrzt_channelx(d_bottom_pump/2+13*px);
                        translate([-(d_bottom_pump/2-xy_vert_channel/2+13*px), 0, 0]) {
                            hrzt_channely(d_bottom_pump/2+xy_vert_channel/2+9*px);
                            translate([0, d_bottom_pump/2+xy_vert_channel/2+9*px, 0]) 
                                vert_channel(0.2+h_hrzt_channel);
                        }
                    }
                }
            }
        }
        translate([-channel_sep/2, 0, -(0.2+h_hrzt_channel)]) {
            mirror([1,0,0]) hrzt_channelx(d_bottom_volume/2-channel_sep/2);
            translate([-(d_bottom_volume/2-channel_sep/2-xy_vert_channel/2), 0, 0]) {
                mirror([0,1,0]) hrzt_channely(d_bottom_volume/2+50*px);
                translate([-xy_vert_channel/2, -(d_bottom_volume/2+50*px), 0]) {
                    hrzt_channelx(d_bottom_volume/2);
                    translate([d_bottom_volume/2, 0, h_hrzt_channel]) {
                        mirror([0,0,1]) vert_channel(0.5+h_hrzt_channel);
                        translate([0,0,-0.2-h_hrzt_channel]) {
                            *mirror([0,0,1]) reservoir();
                            translate([0, -10*px, -0.3]) {
                                rotate([90,0,0]) pinhole(l=1.1);
                                translate([0, 0, -0.07])
                                    hrzt_channely(10*px+xy_vert_channel/2);
                            }
                        }
                    }
                }
            }
        }

        //air channel
        translate([0, -(d_top_volume/2-xy_vert_channel/2), h_bottom_volume+h_top_volume+membranethickness]) {
            vert_channel(1);
            translate([0, 0, 0.4])
                pinhole(l=zbulk-(0.8+valveH*2)-0.14);
        }
        translate([0, d_top_volume/2-xy_vert_channel/2, h_bottom_volume+h_top_volume+membranethickness+0.1]) {
            hrzt_channelx(d_top_volume/2);
            translate([d_top_volume/2-xy_vert_channel/2, 0, 0])
                mirror([0,1,0]) hrzt_channely(120*px+d_top_volume/2-xy_vert_channel/2+0.01);
        }
    }
    
    
    pump();
    //air channel
    translate([0, d_top_pump/2, h_bottom_volume+h_top_volume+membranethickness+0.1]) {
        hrzt_channely(10*px+xy_vert_channel);
        translate([0, 10*px+xy_vert_channel/2, 0]) {
            vert_channel(valveH+h_hrzt_channel+1);
            translate([0, 0, valveH+h_hrzt_channel+0.15])
                pinhole(l=zbulk-(0.8+valveH*2)-0.14);
        }
    }
    translate([0, -d_top_pump/2, h_bottom_volume+h_top_volume+membranethickness+0.1]) {
        mirror([0,1,0]) hrzt_channely(10*px+xy_vert_channel/2);
        translate([0, -(10*px+xy_vert_channel/2), 0]) {
            vert_channel(valveH);
            translate([0, 0, valveH-h_hrzt_channel])
                mirror([0,1,0]) hrzt_channely(120*px-(d_top_pump/2+10*px+xy_vert_channel/2)+0.01);
        }
    }
}

difference() {
    translate([-xbulk/2, -120*px, 0]) 
        bulk();
    difference() {
        translate([0, 0, 0.35+0.8])
            device();
    }
}