stickout = 1;
px = 0.027;
window_sep = 120*px;
layer = 0.050;
membranethickness = 0.4*layer;
membrane_xseparation = 35*px;

xbulk = 680*px;
ybulk = 170*px;
zbulk = 65*layer;

d_bottom_volume = 40*px;
h_bottom_volume = 0.06;
d_top_volume = 40*px;
h_top_volume = 0.1;

d_pump_top = 40*px;
h_pump_top = 0.08;                
d_pump_bottom = 40*px;
h_pump_bottom = 0.08;
pump_memb = 0.4*layer;

xy_vert_channel = 6*px;
dpinhole = 42*px;
odpinhole = 40*px;
idpinhole = 20*px;

channel_sep = 4*px;

l_hrzt_channel = 24*px;
l_hrzt_channel2 = 40*px;
l_hrzt_channel4 = 10*px;
h_hrzt_channel = 0.15;
l_hrzt_channel3 = 70*px;

module bulk(x,y,z) {
  cube(size=[x,y,z], center=false);
}

module top_volume(d, h) {
    color("red") cylinder(d=d, h=h, $fn=3.14*d/0.01, center=false);
}

module bottom_volume(d, h) {
    color("Dodgerblue") cylinder(d=d, h=h, $fn=3.14*d/0.01, center=false);
}

module vert_channel(xy, z) {
    translate([-xy/2, -xy/2, 0]) 
        cube([xy, xy, z]);
}

module hrzt_channel(x, y, z) {
    translate([0, -y/2, 0]) cube([x, y, z]);
}

module pinhole(d, h) {
    color("white") cylinder(d=d, h=h, $fn=3.14*d/0.01, center=false);
}

module pinhole_2(od, id, h) {
    difference() {
        color("pink") cylinder(d=od, h=h+0.1, $fn=3.14*od/0.01, center=false);
        translate([0,0,-0.5]) cylinder(d=id, h=h+1, $fn=3.14*id/0.01, center=false);
    }
    //translate([-xy_vert_channel/2, -xy_vert_channel/2, 0]) color("pink") cube([xy_vert_channel, xy_vert_channel, h+0.1]);
}

module valve(h_top, d_top, d_bottom, h_bottom, memb=membranethickness) {
    bottom_volume(d=d_bottom, h=h_bottom);
    translate([0, 0, h_bottom+memb]) 
        top_volume(d=d_top, h=h_top);
    
    // top volume channels
    translate([0, d_top_volume/2-xy_vert_channel/2, h_top+h_bottom+memb]) {
        vert_channel(xy=xy_vert_channel, z=0.1+h_hrzt_channel);
        translate([0, l_hrzt_channel, 0.1])
            vert_channel(xy=xy_vert_channel, z=3);
        translate([-xy_vert_channel/2, 0, 0.1])
            cube([xy_vert_channel, l_hrzt_channel, h_hrzt_channel]);
        translate([0, l_hrzt_channel, 0.1+h_hrzt_channel+0.15])
            pinhole(d=dpinhole, h=3);
    }
    
    mirror([0,1,0]) {
        translate([0, d_top_volume/2-xy_vert_channel/2, h_top+h_bottom+memb]) {
            vert_channel(xy=xy_vert_channel, z=0.1+h_hrzt_channel);
            translate([-xy_vert_channel/2, 0, 0.1])
                cube([xy_vert_channel, 84*px, h_hrzt_channel]);
        }
    }
    
    // bottom volume channels
    translate([xy_vert_channel/2+channel_sep/2, 0, -0.2-h_hrzt_channel]) {
        vert_channel(xy=xy_vert_channel, z=0.2+h_hrzt_channel); 
        hrzt_channel(x=l_hrzt_channel2, y=xy_vert_channel, z=h_hrzt_channel);
        translate([l_hrzt_channel2, 0, 0])
            vert_channel(xy=xy_vert_channel, z=3);
        translate([l_hrzt_channel2, 0, h_top+h_bottom+memb + (0.1 + h_hrzt_channel)*2 + 0.2])
            pinhole(d=dpinhole, h=3);
    }
    
    mirror([1,0,0]) 
    translate([xy_vert_channel/2+channel_sep/2, 0, -0.2-h_hrzt_channel]) {
        vert_channel(xy=xy_vert_channel, z=0.2+h_hrzt_channel); 
        hrzt_channel(x=l_hrzt_channel4, y=xy_vert_channel, z=h_hrzt_channel);
        *translate([l_hrzt_channel2, 0, 0])
            vert_channel(xy=xy_vert_channel, z=3);
        *translate([l_hrzt_channel2, 0, h_top+h_bottom+memb + (0.1 + h_hrzt_channel)*2 + 0.2])
            pinhole(d=dpinhole, h=3);
    }
}

module pump() {
    translate([0, 0, pump_memb+h_pump_bottom])
        color("Dodgerblue") top_volume(d=d_pump_top, h=h_pump_top);
    color("red") bottom_volume(d=d_pump_bottom, h=h_pump_bottom);
    
    //channels on bottom
    translate([0, d_pump_bottom/2-xy_vert_channel/2, -(0.1+h_hrzt_channel)]) {
        vert_channel(xy=xy_vert_channel, z=0.1+h_hrzt_channel);
        translate([0, l_hrzt_channel3, 0])
            vert_channel(xy=xy_vert_channel, z=3);
        translate([-xy_vert_channel/2, 0, 0])
            cube([xy_vert_channel, l_hrzt_channel3, h_hrzt_channel]);
        translate([0, l_hrzt_channel3, h_hrzt_channel+0.15])
            pinhole(d=dpinhole, h=3);
    }
    
    mirror([0,-1,0]) {
        translate([0, d_pump_bottom/2-xy_vert_channel/2, -(0.1+h_hrzt_channel)]) {
            vert_channel(xy=xy_vert_channel, z=0.1+h_hrzt_channel);
            translate([-xy_vert_channel/2, 0, 0])
                cube([xy_vert_channel, 84*px, h_hrzt_channel]);
        }
    }
}


module device() {
translate([0,0,h_hrzt_channel*2+0.4+h_pump_top+ h_pump_bottom+pump_memb+0.1]) {
    translate([0, 0, -(h_pump_top+h_pump_bottom+pump_memb+h_hrzt_channel+0.4)]) {
        pump();
    
        translate([d_pump_top/2-xy_vert_channel/2, 0, h_pump_bottom+h_pump_top+pump_memb])
            vert_channel(xy=xy_vert_channel, z=h_hrzt_channel+0.2); 
        
        mirror([1,0,0])
        translate([d_pump_top/2-xy_vert_channel/2, 0, h_pump_bottom+h_pump_top+pump_memb])
            vert_channel(xy=xy_vert_channel, z=h_hrzt_channel+0.2); 
    }
    
    translate([d_pump_top/2+10*px, 0, 0]) {
        valve(d_top=d_top_volume, h_top=h_top_volume, d_bottom=d_bottom_volume, h_bottom=h_bottom_volume, memb=membranethickness);
        *mirror([1,0,0]) { 
            translate([xy_vert_channel/2+channel_sep/2, 0, -0.2-h_hrzt_channel]) {
                translate([0, 0, -0.2])
                    vert_channel(xy=xy_vert_channel, z=h_hrzt_channel+0.4); 
            }
        }
    }

    mirror([1,0,0])
    translate([d_pump_top/2+10*px, 0, 0]) {
        valve(d_top=d_top_volume, h_top=h_top_volume, d_bottom=d_bottom_volume, h_bottom=h_bottom_volume, memb=membranethickness);
        *mirror([1,0,0]) { 
            translate([xy_vert_channel/2+channel_sep/2, 0, -0.2-h_hrzt_channel]) {
                translate([0, 0, -0.2])
                    vert_channel(xy=xy_vert_channel, z=h_hrzt_channel+0.4); 
            }
        }
    }
}
}

pumpH = h_top_volume+h_bottom_volume+membranethickness+h_hrzt_channel*3+0.4+h_pump_top+ h_pump_bottom+pump_memb+0.15;
deviceW = (d_pump_top/2+10*px + xy_vert_channel/2 + channel_sep/2 + l_hrzt_channel2 + dpinhole/2)*2;

difference() {
    translate([-120*px, -100*px, -0.5])
        cube([240*px, 250*px, 2.99]);
    difference() {
        device();
        translate([-5, -5, 2.5]) color("lightgrey") cube([10,10,5]);
    }
}